local Media = LibStub("LibSharedMedia-3.0")
local _, PlayerClass = UnitClass("player")
local path = "Interface\\Addons\\TidyPlates_ThreatPlates\\Artwork\\"
local classColors = { 
SHAMAN 		= "2459FF", 
MAGE 		= "69CCF0",
WARLOCK 	= "9482C9",
HUNTER		= "ABD473",
ROGUE		= "FFF569",
PRIEST		= "FFFFFF",
DRUID 		= "FF7D0A",
DEATHKNIGHT = "C41F3B",
WARRIOR 	= "C79C6E",
PALADIN 	= "F58CBA",
}
-- Target Art
local function CreateTargetFrameArt(frame)
	local icon = frame:CreateTexture(nil, 'OVERYLAY', frame)
	icon:SetTexture(nil)
	icon:SetWidth(256)
	icon:SetHeight(64)
	icon:SetPoint("CENTER",frame,"CENTER", 0, 5)
	frame.bars.health:RegisterEvent("CURSOR_UPDATE")
	return icon
end
-- Cast Text
local function CreateTargetCastText(frame)
	local db = TidyPlatesThreat.db.profile.settings
	local text = frame:CreateFontString(nil, 'ARTWORK', 'GameFontNormalLarge')
	text:SetText(nil)
	text:SetJustifyH("CENTER")
	text:SetJustifyV("CENTER")
	text:SetTextColor(1, 1, 1, 1)
	text:SetFont(Media:Fetch('font', db.specialText2.typeface), db.specialText2.size)
	text:SetPoint("CENTER",frame,"CENTER", (db.offsetx+db.specialText2.x), (db.offsety+db.specialText2.y))
	frame.bars.health:RegisterEvent("CURSOR_UPDATE")
	return text
end
-------------------
-- Threat Widget --
-------------------
local threatpath = "Interface\\Addons\\TidyPlates_ThreatPlates\\Artwork\\"
local function SetFade(frame)
	if GetTime() > frame.FadeTime then
		frame:Hide()
		frame:SetScript("OnUpdate", nil)
	end
end

local function GetThreatSegment(threat)
	if not threat then return nil end
	if threat > 135 then return "HIGHTANK"
	elseif threat > 110 then return "MEDIUMTANK"
	elseif threat > 100 then return "LOWTANK"
	elseif threat > 85 then return "HIGHDPS"
	elseif threat > 55 then return "MEDIUMDPS"
	else return "LOWDPS"  end
end

local function GetRelativeThreat(unit)	
	if not UnitExists(unit) then return end
	local _, group, size, index, unitid
	local playerthreat, leaderthreat, tempthreat, leadername = 0,0,0, nil
	_, _,  playerthreat, _, _ = UnitDetailedThreatSituation("player", unit)
	if HasPetUI() then _, _,  leaderthreat, _, _ = UnitDetailedThreatSituation("pet", unit); leadername = "pet" end
	if UnitInRaid("player") then group = "raid"; size = GetNumRaidMembers() - 1
	elseif UnitInParty("player") then group = "party"; size = GetNumPartyMembers()
	else group = nil end
	if group then
		for index = 1, size do
			unitid = group..index
			_, _, tempthreat , _, _ = UnitDetailedThreatSituation(unitid, unit)
			if tempthreat and tempthreat > leaderthreat then 
				leaderthreat = tempthreat 
				leadername = unitid
				end
		end
	end
	if playerthreat and leaderthreat then
		if playerthreat == 100 then return playerthreat + (100-leaderthreat), nil
		else return playerthreat, leadername end
	else return end
end


---------------
-- Threat Widget
---------------
--  [[
local function UpdateThreatWidget(frame, unitid)
	local db = TidyPlatesThreat.db.profile.threatWidget
	local ThreatArt = {
	HIGHTANK = threatpath..db.theme.."\\".."5",
	MEDIUMTANK = threatpath..db.theme.."\\".."4",
	LOWTANK = threatpath..db.theme.."\\".."3",
	HIGHDPS = threatpath..db.theme.."\\".."2",
	MEDIUMDPS = threatpath..db.theme.."\\".."1",
	LOWDPS = threatpath..db.theme.."\\".."1",
	UNKNOWN = threatpath..db.theme.."\\".."Unknown",
	}
	local widgetScale = 64*(db.size)
	if unitid then
		local threat, topholder = GetRelativeThreat(unitid) 
		local threatsegment = GetThreatSegment(threat)
		
		if threatsegment then
			-- Set Indicator
			frame:Show()
			frame.ThreatIcon:SetTexture(ThreatArt[threatsegment])
			frame.ThreatText:SetText(floor(threat).."%")
			if topholder then 
				frame.TargetText:SetText(UnitName(topholder))
				--local _, class = UnitClass(topholder)
				--local classcolor = RAID_CLASS_COLORS[class]
				--frame.TargetText:SetTextColor(classcolor.r, classcolor.g, classcolor.b) 
			else frame.TargetText:SetText("") end
			-- Set Fading
			frame.FadeTime = GetTime() + db.fadeTime
			frame:SetScript("OnUpdate", SetFade)
		else frame:Hide() end
	elseif (GetTime() > frame.FadeTime) or (not InCombatLockdown()) then frame:Hide() end
	-- Custom Updates
	-- Textures
	frame.ThreatIcon:SetWidth(widgetScale)
	frame.ThreatIcon:SetHeight(widgetScale)
	--frame.ThreatIcon:SetPoint("CENTER",parent,"CENTER", db.X, db.Y)
	-- Threat Text
	frame.ThreatText:SetFont((Media:Fetch('font', db.threatFont)), db.threatText.size, db.threatText.flags)
	--frame.ThreatText:SetPoint(db.threatText.AlignH, frame.ThreatIcon, db.threatText.AlignV , db.threatText.x, db.threatText.y)
	frame.ThreatText:SetWidth(db.threatText.width)
	frame.ThreatText:SetHeight(db.threatText.height)
	-- Target Text
	frame.TargetText:SetFont((Media:Fetch('font', db.targetText.font)), db.targetText.size, db.targetText.flags)
	--frame.TargetText:SetPoint(db.targetText.AlignH,frame.ThreatIcon,db.targetText.AlignV, db.targetText.x, db.targetText.y)
	frame.TargetText:SetWidth(db.targetText.width)
	frame.TargetText:SetHeight(db.targetText.height)
end

local function CreateThreatWidget(parent, showname, hangtime, art)
	local db = TidyPlatesThreat.db.profile.threatWidget
	local widgetScale = 64*(db.size)
	local frame = CreateFrame("Frame", nil, parent)
	-- Icon
	frame.ThreatIcon = frame:CreateTexture(nil, "OVERLAY")
	frame.ThreatIcon:SetPoint("CENTER",parent,"CENTER", db.X, db.Y)
	frame.ThreatIcon:SetWidth(widgetScale)
	frame.ThreatIcon:SetHeight(widgetScale)
	-- Threat Text
	frame.ThreatText = frame:CreateFontString(nil, "OVERLAY")
	frame.ThreatText:SetFont((Media:Fetch('font', db.threatFont)), db.threatText.size, db.threatText.flags)
	frame.ThreatText:SetPoint(db.threatText.AlignH, frame.ThreatIcon, db.threatText.AlignV , db.threatText.x, db.threatText.y)
	frame.ThreatText:SetWidth(db.threatText.width)
	frame.ThreatText:SetHeight(db.threatText.height)
	-- Target Text
	frame.TargetText = frame:CreateFontString(nil, "OVERLAY")
	frame.TargetText:SetFont((Media:Fetch('font', db.targetText.font)), db.targetText.size, db.targetText.flags)
	frame.TargetText:SetPoint(db.targetText.AlignH,frame,db.targetText.AlignV, db.targetText.x, db.targetText.y)
	frame.TargetText:SetWidth(db.targetText.width)
	frame.TargetText:SetHeight(db.targetText.height)
	--frame.TargetText:SetShadowOffset(1, -1)
	--frame.TargetText:SetShadowColor(0,0,0,1)
	-- Setup
	frame.FadeTime = 0
	frame:Hide()
	parent.bars.health:RegisterEvent("CURSOR_UPDATE")
	frame:SetScript("OnEvent", function() ForceUpdate(parent:GetParent()) end)
	frame.UpdateThreat = UpdateThreatWidget
	return frame
end


--
local function OnInitialize(plate)
	plate.TargetArt = CreateTargetFrameArt(plate)
	plate.ThreatWidget = CreateThreatWidget(plate)
	plate.CastText = CreateTargetCastText(plate)
end

local function PlateExtensions(plate, unit)
	local style = SetStyleThreatPlates(unit)
	if not plate.TargetArt then OnInitialize(plate) end
	local db = TidyPlatesThreat.db.profile.settings
	local OffX = db.offsetx
	local OffY = db.offsety
	local targetArt
	-- Set Target Art
	if style ~= "empty" then
		if unit.isTarget and UnitExists("target") and db.showTarget and TidyPlatesOptions.theme == "Threat Plates" then
			targetArt = path.."TP_Target"
		else
			targetArt = nil
		end
	else
		targetArt = nil
	end
	plate.TargetArt:SetPoint("CENTER",plate,"CENTER", (OffX+0), (OffY+5))
	plate.TargetArt:SetTexture(targetArt)
	-- Cast Text
	if not plate.CastText then OnInitialize(plate) end

	-- Threat Widget
	if not TidyPlatesThreat.db.profile.threatWidget.ON and ((not unit.isElite) or unit.reaction == "FRIENDLY") then 
		return 
	end
	
	if not plate.ThreatWidget then OnInitialize(plate) end
	
	local unitid = nil
	
	if unit.isMouseover then 
		plate.ThreatWidget:UpdateThreat("mouseover")
	elseif unit.alpha == 1 and UnitExists("target") then 
		plate.ThreatWidget:UpdateThreat("target")
	else 
		plate.ThreatWidget:UpdateThreat(nil) 
	end
end

local setExtended = CreateFrame("Frame")
local themeEvent = {}

function themeEvent:ADDON_LOADED()
	if arg1 == "TidyPlates_ThreatPlates" then
		TidyPlatesThemeList["Threat Plates"].OnInitialize = OnInitialize
		TidyPlatesThemeList["Threat Plates"].OnUpdate = PlateExtensions
	end
end

setExtended:SetScript("OnEvent", function(self, event) themeEvent[event]() end)
setExtended:RegisterEvent("ADDON_LOADED")